ArenaHistory.UI = {}
local UI = ArenaHistory.UI
local Logger = ArenaHistory.Logger
local TeamSort = ArenaHistory.TeamSort
local GameDialog = ArenaHistory.GameDialog
local Player = ArenaHistory.Player
local Filter = ArenaHistory.Filter

UI.logger = Logger.new("UI")

UI.filter = nil
UI.frame = ArenaHistory_UI
UI.gamesContainer = nil
UI.gamesScroll = ArenaHistory_UI_Games
UI.filterOptions = nil

UI.FRAME_WIDTH = 850
UI.FRAME_HEIGHT = 650

UI.ROW_WIDTH = 800
UI.ROW_HEIGHT = 32
UI.NUM_ROWS = 11
UI.SPEC_ICON_SIZE = 32
UI.ROW_PADDING = 16
UI.SCROLL_CONTAINER_HEIGHT = (UI.ROW_HEIGHT + UI.ROW_PADDING) * UI.NUM_ROWS

function UI.applyFilter()
  local self = UI

  local team_size = {}
  for _, bracket in pairs({2, 3, 5}) do
    team_size[bracket] = _G[
        "ArenaHistory_UI_Filter_" .. bracket .. "v" .. bracket]:GetChecked()
    self.logger:debug("Filter by size %d, is enabled: %d", bracket,
        team_size[bracket] and 1 or 0)
  end
  if self.filter ~= nil then
    self.filter:dispose()
  end
  local playerName = ArenaHistory_UI_Filter_Player:GetText()
  self.logger:debug("Filter by player %s", playerName)

  self.filter = Filter.new(function(game)
    if table.getn(game.my_team) ~= table.getn(game.opposite_team) then
      return false
    end
    if not team_size[table.getn(game.my_team)] then
      return false
    end
    if playerName ~= "" then
      local hasPlayer = false
      local lowerPlayerName = string.lower(playerName)
      for _, team in pairs({game.my_team, game.opposite_team}) do
        for _, player in pairs(team) do
          if player.name ~= nil and
              string.lower(string.sub(player.name, 1, string.len(lowerPlayerName))) == lowerPlayerName then
            hasPlayer = true
            break
          end
        end
        if hasPlayer then
          break
        end
      end
      if not hasPlayer then
        return false
      end
    end
    return true
  end)
  self.filter:updateCache(self.NUM_ROWS)
  FauxScrollFrame_OnVerticalScroll(self.gamesScroll, 0,
      ArenaHistory.UI.ROW_HEIGHT + ArenaHistory.UI.ROW_PADDING,
      ArenaHistory.UI.scrollFrameUpdate);
end

function UI:createFilterOptions(container)
  local CHECKBOX_WIDTH = 50

  local frame = CreateFrame("Frame", nil, container)
  frame:SetHeight(self.ROW_HEIGHT)
  frame:SetWidth(self.ROW_WIDTH)

  local filterBy = frame:CreateFontString(nil, nil, "GameFontNormal")
  filterBy:SetText("Filter by:")
  filterBy:SetPoint("TOPLEFT", frame, "TOPLEFT")
  filterBy:SetHeight(20)
  filterBy:Show()

  local align = nil
  for _, bracket in pairs({"2v2", "3v3", "5v5"}) do
    local checkbox = CreateFrame("CheckButton",
        "ArenaHistory_UI_Filter_" .. bracket, frame,
        "ChatConfigCheckButtonTemplate")
    if align == nil then
      checkbox:SetPoint("LEFT", frame, "LEFT")
    else
      checkbox:SetPoint("LEFT", align, "RIGHT", 20, 0)
    end
    checkbox:SetPoint("TOP", filterBy, "BOTTOM", -20, 0)
    checkbox:SetChecked(true)
    local textBox = _G[checkbox:GetName().."Text"]
    textBox:SetText(bracket)
    checkbox:SetHitRectInsets(0, -textBox:GetWidth(), 0, 0)
    checkbox:SetScript("OnClick", UI.applyFilter)
    checkbox:Show()

    align = textBox
  end

  local playerLabel = frame:CreateFontString(nil, nil, "GameFontNormal")
  playerLabel:SetText("Player:")
  playerLabel:SetPoint("TOPLEFT", align, "TOPRIGHT", 30, 0)
  playerLabel:Show()

  local playerInput = CreateFrame("EditBox", "ArenaHistory_UI_Filter_Player",
      frame, "InputBoxTemplate")
  playerInput:SetWidth(150)
  playerInput:SetHeight(20)
  playerInput:SetPoint("LEFT", playerLabel, "RIGHT", 20, 0)
  playerInput:SetPoint("TOP", filterBy, "BOTTOM")
  playerInput:SetScript("OnEnterPressed", UI.applyFilter)
  playerInput:Show()

  local playerClear = CreateFrame("Button", nil, frame, "UIPanelCloseButton")
  playerClear:SetPoint("TOPLEFT", playerInput, "TOPRIGHT", -5,
      -(playerInput:GetHeight() - playerClear:GetHeight()) / 2)
  playerClear:SetScript("OnClick", function()
    if playerInput:GetText() ~= "" then
      playerInput:SetText("")
      UI.applyFilter()
    end
  end)
  playerClear:Show()
 
  frame:Show()
  return frame
end

function UI:createGameScore(game, container)
  local score = container:CreateFontString(nil, nil, "GameFontNormal")
  score:SetTextColor(1, 1, 1)
  score:SetWidth(70)
  if game.score == 1 then
    score:SetTextColor(0, 255, 0, 1)
  end
  if game.score == -1 then
    score:SetTextColor(255, 0, 0, 1)
  end
  if game.my_team_mmr == nil then
    if game.score == 1 then
      score:SetText("won")
    elseif game.score == -1 then
      score:SetText("lost")
    else
      score:SetText("draw")
    end
  else
    score:SetText(string.format("%d", game.my_team_mmr))
  end
  score:Show()
  return score
end

function UI:createSpec(player, container)
  local icon = container:CreateTexture()
  icon:SetWidth(self.SPEC_ICON_SIZE)
  icon:SetHeight(self.SPEC_ICON_SIZE)
  icon:SetNonBlocking(true)
  local cut = 0.08
  icon:SetTexCoord(cut, 1 - cut, cut, 1 - cut)
  if player.spec_id ~= nil then
    local iconPath = select(4,
        GetSpecializationInfoByID(player.spec_id))
    if iconPath ~= nil then
      icon:SetTexture(iconPath)
    end
  end
  icon:Show()
  return icon
end

function UI:createDateAndTime(game, container)
  local label = container:CreateFontString(nil, nil, "GameFontNormal")
  label:SetTextColor(1, 1, 1)
  if game.region == "US" then
    label:SetText(date("%m/%d/%y %H:%M", game.start_timestamp))
  else
    label:SetText(date("%d/%m/%y %H:%M", game.start_timestamp))
  end
  label:Show()
  return label
end

function UI:createFirstDied(game, container, align)
  local text = container:CreateFontString(nil, nil, "GameFontNormal")
  local player = nil
  if (Player.isArenaId(game.first_died_id)) then
    text:SetText("You killed:")
    for _, p in pairs(game.opposite_team) do
      if p.id == game.first_died_id then
        player = p
        break
      end
    end
  else
    text:SetText("They killed:")
    for _, p in pairs(game.my_team) do
      if p.id == game.first_died_id then
        player = p
        break
      end
    end
  end
  if player == nil then
    return align
  end
  text:SetTextColor(1, 1, 1)
  text:SetPoint("LEFT", align, "RIGHT", 20, 0)
  text:Show()
 
  local spec = self:createSpec(player, container)
  spec:SetPoint("LEFT", text, "RIGHT", 5, 0)

  return spec
end

function UI:createGame(game, container, is_clickable)
  local gameContainer = CreateFrame("Frame", nil, container)
  gameContainer:SetWidth(self.ROW_WIDTH)
  gameContainer:SetHeight(self.ROW_HEIGHT)

  if game == nil then
    return gameContainer
  end

  gameContainer:EnableMouse(true)
  if is_clickable then
    local hover = gameContainer:CreateTexture(nil, "HIGHLIGHT")
    hover:SetAllPoints(true)
    hover:SetTexture(1, 1, 1, 0.1)
    hover:Show()
  end

  if is_clickable then
    gameContainer:SetScript("OnMouseUp", function()
      GameDialog:show(game)
      UI:hide()
    end)
  end

  -- Date
  local dateLabel = self:createDateAndTime(game, gameContainer) 
  dateLabel:SetPoint("LEFT", gameContainer, "LEFT")

  -- Score
  local score = self:createGameScore(game, gameContainer)
  score:SetPoint("LEFT", dateLabel, "RIGHT")

  -- My team
  TeamSort.sort(game.my_team)
  local align = nil
  for _, player in pairs(game.my_team) do
    local spec = self:createSpec(player, gameContainer)
    if align ~= nil then
      spec:SetPoint("LEFT", align, "RIGHT", 5, 0)
    else
      spec:SetPoint("LEFT", score, "RIGHT")
    end
    align = spec
  end

  -- "vs"
  local vs = gameContainer:CreateFontString(nil, nil, "GameFontNormal")
  vs:SetText("vs")
  vs:SetTextColor(1, 1, 1)
  vs:SetPoint("LEFT", align, "RIGHT", 10, 0)
  vs:Show()

  -- Opposite team
  TeamSort.sort(game.opposite_team)
  align = nil
  for _, player in pairs(game.opposite_team) do
    local spec = self:createSpec(player, gameContainer)
    if align ~= nil then
      spec:SetPoint("LEFT", align, "RIGHT", 5, 0)
    else
      spec:SetPoint("LEFT", vs, "RIGHT", 10, 0)
    end
    align = spec
  end

  if align == nil then
    align = vs
  end

  -- Hwo died first
  if game.first_died_id ~= nil then
    local firstDied = self:createFirstDied(game, gameContainer, align)
    align = firstDied    
  end

  -- Run time
  if game.run_time_s ~= nil then
    local runTime = gameContainer:CreateFontString(nil, nil, "GameFontNormal")
    runTime:SetText(string.format("%02d:%02d",
        math.floor(game.run_time_s / 60),
        game.run_time_s % 60))
    runTime:SetTextColor(1, 1, 1)
    runTime:SetPoint("LEFT", align, "RIGHT", 20, 0)
    runTime:Show()
    align = runTime
  end

  -- Map name
  if game.localized_map_name ~= nil then
    local map = gameContainer:CreateFontString(nil, nil, "GameFontNormal")
    map:SetText(game.localized_map_name)
    map:SetTextColor(1, 1, 1)
    map:SetPoint("LEFT", align, "RIGHT", 20, 0)
    map:Show()
    align = map
  end

  gameContainer:Show()
  return gameContainer
end

function UI:createGamesList(left, right, container)
  local gamesList = CreateFrame("Frame", nil, container)
  gamesList:SetHeight(self.SCROLL_CONTAINER_HEIGHT)
  gamesList:SetWidth(self.ROW_WIDTH)

  local align = nil
  for i = left, right do
    local game = self.filter:getGame(i)
    local row = self:createGame(game, gamesList, true)
    if align ~= nil then
      row:SetPoint("TOP", align, "BOTTOM", 0, -self.ROW_PADDING)
    else
      row:SetPoint("TOP", gamesList, "TOP", 0, -self.ROW_PADDING)
    end
    row:SetPoint("LEFT", gamesList, "LEFT")
    align = row
  end
  gamesList:Show()
  return gamesList 
end

function UI:addGames(left, right)
  if self.gamesContainer ~= nil then
    self.gamesContainer:Hide()
    self.gamesContainer:SetParent(nil)
    self.gamesContainer = nil
  end
  self.gamesContainer = self:createGamesList(left, right, self.frame)
  self.gamesContainer:SetPoint("TOPLEFT", self.gamesScroll, "TOPLEFT")
end

function UI.scrollFrameUpdate()
  local self = UI
  local maxGames = self.filter:getCount(self.NUM_ROWS + 1)
  FauxScrollFrame_Update(self.gamesScroll, maxGames + 1,
      self.NUM_ROWS, self.ROW_HEIGHT + self.ROW_PADDING);
  local offset = FauxScrollFrame_GetOffset(self.gamesScroll)
  local leftIndex = 1 + offset
  if leftIndex > maxGames then
    leftIndex = maxGames
  end
  local rightIndex = self.NUM_ROWS + offset
  if rightIndex > maxGames then
    rightIndex = maxGames
  end
  self:addGames(leftIndex, rightIndex)
  local maxGames = self.filter:getCount(self.NUM_ROWS + 1)
  FauxScrollFrame_Update(self.gamesScroll, maxGames + 1,
      self.NUM_ROWS, self.ROW_HEIGHT + self.ROW_PADDING);
end

function UI:show()
  if self.filterOptions == nil then
    self.filterOptions = self:createFilterOptions(self.frame)
    self.filterOptions:SetPoint("TOPLEFT", self.frame, "TOPLEFT", 30, -38)
  end
  if self.filter == nil then
    self.filter = Filter.new(
        function(game)
          return true
        end)
    self.filter:updateCache(self.NUM_ROWS)
  end
  self.frame:SetWidth(self.FRAME_WIDTH)
  self.frame:SetHeight(self.FRAME_HEIGHT)
  self.gamesScroll:SetPoint("TOPLEFT", self.filterOptions, "BOTTOMLEFT", 0, -10)
  self.gamesScroll:SetPoint("BOTTOMRIGHT", self.frame, "BOTTOMRIGHT", -30, 24)
  self.gamesScroll:Show()
  self.frame:Show()
end

function UI:hide()
  self.frame:Hide()
end

function UI:toggle()
  if self.frame:IsShown() then
    self:hide()
  else
    GameDialog:hide()
    self:show()
  end
end
